/*
 * Written by Oron Peled <oron@actcom.co.il>
 *
 * Modified by Dominique Domet de Mont, 2007, Dominique.Domet-de-Mont@hp.com
 * Copyright (C) 2007 Hewlett-Packard Development Company, L.P.
 *
 * Some code snippets adapted from spak
 * (http://www.xenos.net/software/spak/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <net/if.h>
#include <sys/select.h>
#include "config.h"
#include "linkloop.h"

/*
 * This program runs as a daemon, listening at the llc level on network interfaces 
 * passed on the command line, and responding by sending back a test packet.
 */
 
static char	*program = NULL;

u_int8_t	mac_src[MAX_IFACES][IFHWADDRLEN];
u_int8_t	mac_dst[IFHWADDRLEN];

int main(int argc, char *argv[]) {
	int sock[MAX_IFACES];
	size_t len;
	struct llc_packet spack;
	struct llc_packet rpack;
	fd_set rfds;	/* the set of file descriptors to listen */
    int retval;		/* select returned value */
    int maxfd = 0;	/* total number of file descriptors */
    int nif = argc -1; 	/* number of interfaces to listen */
    int i = nif;	/* loop iterator */
	
	program = argv[0];
	FD_ZERO(&rfds);

	if(nif == 0 || nif > MAX_IFACES)
	{
		/* one parameter expected at least: the lan interface name to listen */
		printf("Expecting a list of one to %d interfaces to listen; eg: %s eth0 eth3\n", MAX_IFACES, program);
		return 1;
	}

	/* Open the sockets */
	for(; i--;)
	{
		printf ("Opening a socket on %s\n", argv[i+1]);
		if ((sock[i] = socket(AF_INET, SOCK_PACKET, htons(ETH_P_802_2))) == -1) {
			perror("socket");
			return 1;
		}
		get_hwaddr(sock[i], argv[i+1], mac_src[i]);
		FD_SET(sock[i], &rfds);
		if(maxfd < sock[i])maxfd = sock[i];
	}

	/* listen and reply forever */
	do {
		retval = select(maxfd+1, &rfds, NULL, NULL, NULL);
		if (retval == -1)
		{
			perror("select()");
			continue;
		}
		
		/* check all file descriptors */
		for(i = nif; i--;)
		{
			if(!FD_ISSET(sock[i], &rfds))
				continue;
				
			printf("Received packet on %s\n", argv[i+1]);
			len = recv_packet(sock[i], &rpack);
			memcpy(mac_dst, rpack.eth_hdr.ether_shost, IFHWADDRLEN);
			mk_test_packet(&spack, mac_src[i], mac_dst, len, 1);
	
			send_packet(sock[i], argv[i+1], &spack);
		}
	} while(1);
	return 0;
}
